package de.isse.kiv

import java.rmi.AlreadyBoundException
import java.rmi.RemoteException
import java.rmi.registry.LocateRegistry
import java.rmi.registry.Registry
import org.eclipse.core.resources.ResourcesPlugin
import org.eclipse.core.runtime.preferences.IEclipsePreferences
import org.eclipse.core.runtime.preferences.InstanceScope
import org.eclipse.jface.resource.ImageDescriptor
import org.eclipse.ui.IStartup
import org.eclipse.ui.plugin.AbstractUIPlugin
import org.osgi.framework.BundleContext
import KIVPlugin._
import java.net.URL
import kiv.communication.TheEventQueue
import kiv.gui.dialogFactory

object KIVPlugin {
  val PLUGIN_URL = "platform:/plugin/kiv.editor"
  val PLUGIN_ID = "de.isse.kiv.editor"
  val IMPORT_WIZARD_ID = "de.isse.kiv.ui.wizards.importProject"

  lazy val getDefault = new KIVPlugin()
}

class KIVPlugin extends AbstractUIPlugin with IStartup {
  val watch = new DevgraphWatch()

  override def start(context: BundleContext) {
    super.start(context)    
    watch.start()

    // Reroute KIV <-> KIV GUI communication to exceptions
    dialogFactory.setDialog(KIVDialog)
  }

  override def stop(context: BundleContext) {
    watch.stop()
    super.stop(context)
  }

  private def activatePolling() {
    val prefs = InstanceScope.INSTANCE.getNode("org.eclipse.core.resources")
    prefs.putBoolean(ResourcesPlugin.PREF_AUTO_REFRESH, true)
  }

  override def earlyStartup() {
    activatePolling()
    KIVEditorServer.bind()
  }
}
