package de.isse.kiv.refactoring

import org.eclipse.core.resources.IFile
import org.eclipse.core.runtime.IProgressMonitor
import org.eclipse.ltk.core.refactoring.Change
import org.eclipse.ltk.core.refactoring.Refactoring
import org.eclipse.ltk.core.refactoring.RefactoringStatus
import org.eclipse.ui.texteditor.ITextEditor
import org.eclipse.ltk.core.refactoring.CompositeChange
import org.eclipse.jface.text.IDocument
import org.eclipse.ui.texteditor.IDocumentProvider
import org.eclipse.ui.editors.text.TextFileDocumentProvider
import de.isse.kiv.resources.ResourceLookup
import scala.collection.mutable.ListBuffer
import de.isse.kiv.resources.ResourceProperties._
import de.isse.kiv.resources.ProjectModel
import org.eclipse.core.filesystem.IFileStore
import kiv.signature.Sigentry
import kiv.signature.Csignature
import kiv.basic.Typeerror
import kiv.expr.Xov
import de.isse.kiv.resources.FileModel
import kiv.parser._
import de.isse.kiv.source.parser._
import de.isse.kiv.source.TokenMap

class CodeRenameRefactoring(editor: ITextEditor, kivToken: KIVToken, file: IFile, fileModel: FileModel) extends Refactoring {

  var selectedTokenString: String = kivToken.stringPresentation
  filterString(file.specName)
  var editedString: String = _
  private val sigentry: Sigentry = (kivToken: @unchecked) match {
    case VariableToken(prexov: PreXov)    => prexov.xov
    case ProcedureToken(preproc: PreProc) => preproc.proc
    case OperationToken(preop: PreOp)     => preop.op
    // TODO: OperationDefinitionToken
  }

  var updateReferences: Boolean = _
  private var csignatureWithDefinitionOfSigentry: Csignature = _
  private var counter: Integer = 0;
  private var renameListBuffer: ListBuffer[RenameTextChange] = _;

  def checkFinalConditions(x$1: IProgressMonitor): RefactoringStatus = {
    val refactoringStatus = new RefactoringStatus
    refactoringStatus
  }

  def checkInitialConditions(x$1: IProgressMonitor): RefactoringStatus = {
    val refactoringStatus = new RefactoringStatus

    if (kivToken.canRename) {
      refactoringStatus.addInfo("Nothing to rename")
      return refactoringStatus
    }

    //TODO: Add information to the Status
    refactoringStatus
  }

  /**
   *  This is the "preview" change in the wizard
   *  The actual change occures when the user clicks the wizards okay button
   */
  def createChange(monitor: IProgressMonitor): Change = {
    monitor.beginTask("Creating change...", IProgressMonitor.UNKNOWN);
    getChangeObject()
  }

  def getChangeObject(): Change = {
    val docProv = editor.getDocumentProvider
    if (docProv == null) return null
    val document = docProv.getDocument(editor.getEditorInput)

    renameListBuffer = new ListBuffer[RenameTextChange]();

    val projectModel = ResourceLookup.activeProject.projectModel

    val changeFiles: List[IFile] = if (updateReferences) {
      val csignatureOfOriginFileContainsSigentry = csignautreOfFileContainsSigentry(file);
      println("Csignature of origin file contains sigentry: " + csignatureOfOriginFileContainsSigentry)
      if (csignatureOfOriginFileContainsSigentry) {
        (getAllFilesFromSubSpecs(projectModel.subSpecsReversed, file.specName) ++ List(file) ++ getOtherRelatedFile(file)).distinct
      } else {
        val definitionFile = getFileWithDefinitionOfSigentry(projectModel, file.specName)
        println("Found other definition file than origin file: " + definitionFile)
        definitionFile match {
          case Some(dFile) => (getAllFilesFromSubSpecs(projectModel.subSpecsReversed, dFile.specName) ++ List(dFile) ++ getOtherRelatedFile(dFile)).distinct
          case None        => Nil
        }
      }
    } else {
      List(file)
    }

    changeFiles.foreach { cFile =>
      if (getLocationsFromFile(cFile) != Nil)
        renameListBuffer += new RenameTextChange(getDocumentFromFile(cFile), getLocationsFromFile(cFile), kivToken, editedString)
    }

    val csignature = csignatureWithDefinitionOfSigentry
    var loop: Boolean = true;

    changeFiles.foreach { cfile => cfile.fileModel.tokenizeTokensForFile(cfile) }
    if (csignature != null) sigentry match {
      case xov: Xov => {
        while (loop) {
          val sigentry = new Xov(Symbol(selectedTokenString + counter), xov.typ, xov.flexiblep); val editedStringExtra = editedString + counter
          if (varlist(csignature).contains(sigentry)) {
            println("new RenameTextChange for: " + counter + " changed name: " + editedStringExtra)
            changeFiles.foreach { cFile => println("Locations from cFile: " + getLocationsFromFile(cFile, sigentry)) }
            changeFiles.foreach { cFile =>
              if (getLocationsFromFile(cFile, sigentry) != Nil)
                addOtherLocations(cFile, sigentry, editedStringExtra)
            }
            counter = counter + 1
          } else loop = false;
        }
      }
      case _ =>
    }
    val change = new CompositeChange("composite", renameListBuffer.toArray)
    change
  }

  def addOtherLocations(cFile: IFile, sigentry: Sigentry, editedStringExtra: String) = {
    if (!renameListBuffer.filter { rtc => rtc.document == getDocumentFromFile(cFile) }.isEmpty)
      renameListBuffer.foreach { rtc => if (rtc.document == getDocumentFromFile(cFile)) rtc.addEdits(getLocationsFromFile(cFile, sigentry), editedStringExtra) }
    else renameListBuffer += new RenameTextChange(getDocumentFromFile(cFile), getLocationsFromFile(cFile, sigentry), kivToken, editedStringExtra)
  }

  def getOtherRelatedFile(f: IFile): List[IFile] = {
    val filemodel = f.fileModel
    val spec = filemodel.getPreSpecificationOrSequents
    spec match {
      case Some(Left(prespec)) => getSequentsFromSpecFile(f) match {
        case Some(file) => List(file)
        case None       => Nil
      }
      case Some(Right(theorems)) => getSpecFromSequentsFile(f) match {
        case Some(file) => List(file)
        case None       => Nil
      }
      case None => Nil
    }
  }

  def getSequentsFromSpecFile(f: IFile): Option[IFile] = {
    ResourceLookup.seqFileOrLibrary(f.specName) match {
      case Left(file: IFile)        => Some(file)
      case Right(store: IFileStore) => println("Sequents file nicht verfügbar! FileStoreobjekt"); None
    }
  }

  def getSpecFromSequentsFile(f: IFile): Option[IFile] = {
    ResourceLookup.specFileOrLibrary(f.specName) match {
      case Left(file: IFile)        => Some(file)
      case Right(store: IFileStore) => println("Spec file nicht verfügbar! FileStoreobjekt"); None
    }
  }

  def filterString(specName: String) = {
    val newSpecName = specName.replaceAll("-", "_") + "_"
    println(newSpecName)
    fileModel.getPreSpecificationOrSequents match {
      case Some(Left(prespec: PreDataASMSpecification)) =>
        selectedTokenString = selectedTokenString.replace(newSpecName, ""); println(selectedTokenString)
      case _ =>
    }
  }

  def getDocumentFromFile(ifile: IFile): IDocument = {
    val provider: IDocumentProvider = new TextFileDocumentProvider();
    provider.connect(ifile);
    val document = provider.getDocument(ifile)
    document
  }

  def getLocationsFromFile(ifile: IFile): List[Location] = {
    val filemodel = ifile.fileModel
    val analyzer = filemodel.getSpecAnalyzer
    analyzer match {
      case Some(analyzer) => analyzer.getLocationsFromSigentry(sigentry)
      case None           => Nil
    }
  }

  def getLocationsFromFile(ifile: IFile, sigentry: Sigentry): List[Location] = {
    val filemodel = ifile.fileModel
    val analyzer = filemodel.getSpecAnalyzer
    analyzer match {
      case Some(analyzer) => analyzer.getLocationsFromSigentry(sigentry)
      case None           => Nil
    }
  }

  def getAllFilesFromSubSpecs(map: Map[String, List[String]], name: String): List[IFile] = {
    val subSpecs = map(name)
    if (subSpecs.isEmpty) getFilesFromSpecNames(subSpecs)
    else getFilesFromSpecNames(subSpecs) ++ subSpecs.flatMap { subSpec => getAllFilesFromSubSpecs(map, subSpec) }
  }

  def getFilesFromSpecNames(names: List[String]): List[IFile] = {
    val filesOrLibraries = names.map { name => ResourceLookup.specFileOrLibrary(name) } ++ names.map { name => ResourceLookup.seqFileOrLibrary(name) }
    var listbuffer = new ListBuffer[IFile]()
    filesOrLibraries.foreach(fileOrLibrary => fileOrLibrary match {
      case Left(file)       => listbuffer += file
      case Right(filestore) =>
    })
    listbuffer.toList
  }

  def getFileWithDefinitionOfSigentry(projectModel: ProjectModel, name: String): Option[IFile] = {
    val subSpecsNames = projectModel.subSpecs(name)
    val files = getFilesFromSpecNames(subSpecsNames)
    val filteredFiles = files.filter { file => csignautreOfFileContainsSigentry(file) }
    if (filteredFiles.isEmpty) {
      if (subSpecsNames.isEmpty) None
      else getFileWithDefinitionOfSigentry(projectModel, subSpecsNames)
    } else Some(filteredFiles.head)
  }

  def getFileWithDefinitionOfSigentry(projectModel: ProjectModel, names: List[String]): Option[IFile] = {
    val files = names.flatMap { name => getFilesFromSpecNames(projectModel.subSpecs(name)) }
    val filteredFiles = files.filter { file => csignautreOfFileContainsSigentry(file) }
    if (filteredFiles.isEmpty) {
      val subSpecs = names.flatMap { name => projectModel.subSpecs(name) }.distinct
      if (subSpecs.isEmpty) None
      else getFileWithDefinitionOfSigentry(projectModel, subSpecs)
    } else Some(filteredFiles.head)
  }

  def csignautreOfFileContainsSigentry(file: IFile): Boolean = {
    val fileModel = file.fileModel
    val spec = fileModel.getPreSpecificationOrSequents
    spec match {
      case Some(Left(prespec)) => fileModel.getSpec match {
        case Some(spec) => {
          try {
            val csignature = spec.csignature
            println(file.specName + " " + csignature)
            if (varlist(csignature).contains(sigentry) || oplist(csignature).contains(sigentry) || proclist(csignature).contains(sigentry)) {
              csignatureWithDefinitionOfSigentry = csignature; true
            }
            else false
          } catch { case typeerror: Typeerror => println(file.specName + " has no csignature"); false }
        }
        case _ => false
      }

      case Some(Right(theorems)) =>
        (file.isSequents && csignautreOfFileContainsSigentry(getSpecFromSequentsFile(file).get))

      case _ => fileModel.getSpec match {
        case Some(spec) => {
          try {
            val csignature = spec.csignature
            varlist(csignature).contains(sigentry) || oplist(csignature).contains(sigentry) || proclist(csignature).contains(sigentry)
          } catch { case _: Typeerror => false }
        }
        case None => println(file.specName + " has no spec!"); false
      }
    }
  }

  private def convertToOldProc(projectModel: ProjectModel, procname: String, ifile: IFile): String = {
    val procMap = projectModel.getProcMap(ifile)
    procMap(procname).procsym.name
  }

  private def varlist(csignature: Csignature) = {
    csignature.varcommentlist.map(varcomment => varcomment._1)
  }

  private def oplist(csignature: Csignature) = {
    csignature.combinedOpList.map(opcomment => opcomment._1)
  }

  private def proclist(csignature: Csignature) = {
    csignature.proccommentlist.map(proccomment => proccomment._1)
  }

  def getName(): String = "CodeRenameRefactoring"
}
