package de.isse.kiv.refactoring

import org.eclipse.jface.text.templates.Template
import org.eclipse.jface.text.templates.TemplateContextType
import org.eclipse.ui.texteditor.ITextEditor
import kiv.parser.Terminals._
import kiv.printer.prettyprint.xpp
import org.eclipse.jface.text.templates.DocumentTemplateContext
import org.eclipse.jface.text.templates.TemplateProposal
import org.eclipse.jface.text.Region
import kiv.expr.Op
import kiv.expr.variables.get_new_vars_for_sorts
import kiv.expr.Funtype
import kiv.spec.Gen
import kiv.expr.Ap
import org.eclipse.jface.text.ITextViewer
import kiv.expr.Xov
import kiv.expr.Type
import kiv.expr.Expr
import de.isse.kiv.refactoring.KIVRefactoring._
import de.isse.kiv.source.WithSpec

object KIVRefactoring {
  def newVars(ts: List[Type], forbs: List[Xov], specvars: List[Xov]) = {
    get_new_vars_for_sorts(ts, ts.map { _ => false }, forbs, specvars)
  }
}

class KIVRefactoring(val editor: ITextEditor, viewer: ITextViewer) extends WithSpec {
  def document = viewer.getDocument

  def addToSection(id: Short, text: String) {

    fileModel.axiomToken map {
      case kivToken =>
        val offset = kivToken.location.position + kivToken.location.length
        val length = 0
        val region = new Region(offset, length)
        val template = new Template("", "", "", "\n" + text, true)
        val contextType: TemplateContextType = new TemplateContextType("stdType")
        val ctx = new DocumentTemplateContext(contextType, document, offset, length)
        val completion = new TemplateProposal(template, ctx, region, null)
        completion.apply(viewer, 0: Char, 0, 0)
    }
  }

  def addRecursiveDefinition(op: Op, pos: Int) {
    val text = recursiveDefinition(op, pos)
    text.map(addToSection(T_AXIOMS, _))
  }

  def recursiveDefinition(op: Op, pos: Int) = op.typ match {
    case Funtype(ats, res) =>
      val specvars = spec.get.specvars
      val (ts1, rec :: ts2) = ats.splitAt(pos)
      val xs1 = newVars(ts1, Nil, specvars)
      val xs2 = newVars(ts2, xs1, specvars)
      val Gen(_, consts, ops, _) = spec.get.specgens.find(_.gensortlist contains rec).get

      val opargs = ops.map {
        op =>
          val types = op.typ.typelist
          Ap(op.toInstOp, toPatterns(newVars(types, xs1 ++ xs2, specvars)))
      }

      val basecases = consts.map(pat => Ap(op.toInstOp, toPatterns(xs1 ::: pat.toInstOp :: xs2)))
      val reccases = opargs.map(pat => Ap(op.toInstOp, toPatterns(xs1 ::: pat :: xs2)))

      val eqsym = if (op.prdp) "↔" else "="

      val lines1 = basecases.zipWithIndex.map {
        case (ap, i) =>
          val j = i
          xpp(op) + "-base-" + i + ":\n" + xpp(ap) + "\n  " + eqsym + " ${_" + j + "};"
      }

      val n = basecases.length
      val lines2 = reccases.zipWithIndex.map {
        case (ap, i) =>
          val j = i + n
          xpp(op) + "-rec-" + i + ":\n" + xpp(ap) + "\n  " + eqsym + " ${_" + j + "};"
      }

      val lines = lines1 ++ lines2
      Some(lines.mkString("\n\n", "\n\n", "\n"))

    case _ =>
      None
  }

  def toPatterns(xs: List[Expr]) = xs map {
    case Xov(Symbol(name), typ, flexiblep) =>
      Xov(Symbol("${" + name + "}"), typ, flexiblep)
    case expr =>
      expr
  }
}
