package de.isse.kiv.resources

import java.io.ByteArrayInputStream
import java.io.File
import java.io.InputStream
import kiv.fileio.Directory
import kiv.project.Devgraphordummy
import kiv.project.Projectinfo
import org.eclipse.core.resources.ICommand
import org.eclipse.core.resources.IFile
import org.eclipse.core.resources.IFolder
import org.eclipse.core.resources.IProject
import org.eclipse.core.resources.IProjectDescription
import org.eclipse.core.resources.IResource
import org.eclipse.core.resources.IWorkspaceRoot
import org.eclipse.core.resources.ResourcesPlugin
import org.eclipse.core.runtime.CoreException
import org.eclipse.core.runtime.IPath
import org.eclipse.core.runtime.IProgressMonitor
import de.isse.kiv.ui.Console
import de.isse.kiv.project.Builder
import de.isse.kiv.project.Nature

class Import(name: String, comment: String, location: IPath, monitor: IProgressMonitor) {
  val (project, isLocal) = {
    val root = ResourcesPlugin.getWorkspace.getRoot
    val rootPath = root.getLocation
    (root.getProject(name), rootPath.append(name) == location)
  }

  private def createEclipseProject() {
    if (!project.exists()) project.create(monitor)
    if (!project.isOpen) project.open(monitor)
    project.setDefaultCharset("UTF-8", monitor)
    val toplevel = location.toFile()
    if (!toplevel.exists()) toplevel.mkdirs()
  }

  private def initializeNature() {
    val description = project.getDescription
    if (!project.hasNature(Nature.NATURE_ID)) {
      description.setNatureIds(description.getNatureIds ++ Set(Nature.NATURE_ID))
    }
    val cmds = description.getBuildSpec
    val hasBuilder = cmds.exists(_.getBuilderName == Builder.BUILDER_ID)
    if (!hasBuilder) {
      val cmd = description.newCommand()
      cmd.setBuilderName(Builder.BUILDER_ID)
      description.setBuildSpec(cmds ++ Set(cmd))
    }
    project.setDescription(description, monitor)
  }

  private def createKIVFiles() {
    if (location.append(ResourceProperties.DEVGRAPH).toFile.exists) return
    val osLocation = location.toOSString
    val directory = new Directory(osLocation)
    val devgraph = kiv.project.Devgraph.default_devgraph(name).setProjectpath(osLocation)
    devgraph.save_devgraph_dir_til_ok(directory)
    val projectInfo = kiv.project.Projectinfo.default_projectinfo(name).setProjectinfocomment(comment).setProjectinfodate(kiv.util.string.current_date_string()).setProjectinfouserhome(kiv.fileio.file.user_home()).setProjectinfouser(kiv.fileio.file.current_user())
    projectInfo.save_projectinfo_dir_til_ok(directory)
    kiv.gui.file.overwrite_til_ok(kiv.spec.installskels.config_project_skel(), directory.truename + "/" + kiv.fileio.globalfiledirnames.config_file_name)
  }

  private def importFolder(path: IPath, folder: IFolder) {
    if (!folder.exists()) {
      if (isLocal) {
        folder.create(IResource.NONE, isLocal, monitor)
      } else {
        path.toFile().mkdir()
        folder.createLink(path, IResource.FOLDER, monitor)
      }
    } else if (folder.exists() && folder.isLinked) {
      folder.delete(true, monitor)
      folder.createLink(path, IResource.FOLDER, monitor)
    }
  }

  private def importFile(path: IPath, file: IFile) {
    if (!file.exists()) {
      if (path.toFile().exists() && !isLocal) file.createLink(path, IResource.FILE, monitor)
    } else if (file.exists() && file.isLinked) {
      file.delete(true, monitor)
      file.createLink(path, IResource.FILE, monitor)
    }
  }

  private def importKIVFiles() {
    val specs_path = location.append(ResourceProperties.SPECS)
    val specs = project.getFolder(ResourceProperties.SPECS)
    importFolder(specs_path, specs)
    val doc_path = location.append(ResourceProperties.DOC)
    val doc = project.getFolder(ResourceProperties.DOC)
    importFolder(doc_path, doc)
    val devgraph_path = location.append(ResourceProperties.DEVGRAPH)
    val devgraph_file = project.getFile(ResourceProperties.DEVGRAPH)
    importFile(devgraph_path, devgraph_file)
    val config_path = location.append(ResourceProperties.CONFIG)
    val config_file = project.getFile(ResourceProperties.CONFIG)
    importFile(config_path, config_file)
    val project_info_path = location.append(ResourceProperties.PROJECT_INFO)
    val project_info_file = project.getFile(ResourceProperties.PROJECT_INFO)
    importFile(project_info_path, project_info_file)
  }

  def createProject() {
    createEclipseProject()
    initializeNature()
    createKIVFiles()
    project.refreshLocal(IResource.DEPTH_INFINITE, monitor)
    importKIVFiles()
    project.refreshLocal(IResource.DEPTH_INFINITE, monitor)
  }
}
