package de.isse.kiv.resources

import org.eclipse.core.resources.IMarker
import org.eclipse.core.resources.IResource
import org.eclipse.core.runtime.CoreException
import org.eclipse.ui.texteditor.SimpleMarkerAnnotation
import org.eclipse.jface.text.source.Annotation
import org.eclipse.core.resources.ResourcesPlugin

object Marker {
  val ERROR_MARKER_ID = "de.isse.kiv.ui.marker"
  val TEXT_MARKER_ID = "de.isse.kiv.ui.textmarker"
  val WARNING_MARKER_ID = "de.isse.kiv.ui.warningmarker"

  def createErrorMarker(resource: IResource, errormsg: String, typ: String, line: Int, start: Int, end: Int) {
    try {
      val marker = resource.createMarker(ERROR_MARKER_ID)

      marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_ERROR)
      marker.setAttribute(IMarker.MESSAGE, errormsg)
      marker.setAttribute(IMarker.PRIORITY, IMarker.PRIORITY_HIGH)
      marker.setAttribute(IMarker.LOCATION, "line " + (line + 1))
      marker.setAttribute(IMarker.PROBLEM, typ)
      marker.setAttribute(IMarker.CHAR_START, start)
      marker.setAttribute(IMarker.CHAR_END, end)
      marker.setAttribute(IMarker.LINE_NUMBER, line + 1) // NOTE: LINE_NUMBER is 1-based
    } catch {
      case e: CoreException =>
    }
  }
  
  def createWarningMarker(resource: IResource, errormsg: String, typ: String, line: Int, start: Int, end: Int) {
    try {
      val marker = resource.createMarker(WARNING_MARKER_ID)

      marker.setAttribute(IMarker.SEVERITY, IMarker.SEVERITY_WARNING)
      marker.setAttribute(IMarker.MESSAGE, errormsg)
      marker.setAttribute(IMarker.PRIORITY, IMarker.PRIORITY_NORMAL)
      marker.setAttribute(IMarker.LOCATION, "line " + (line + 1))
      marker.setAttribute(IMarker.PROBLEM, typ)
      marker.setAttribute(IMarker.CHAR_START, start)
      marker.setAttribute(IMarker.CHAR_END, end)
      marker.setAttribute(IMarker.LINE_NUMBER, line + 1) // NOTE: LINE_NUMBER is 1-based
    } catch {
      case e: CoreException =>
    }
  }

  def createVariableMarker(resource: IResource, range: Range) {
    try {

      val marker = resource.createMarker(TEXT_MARKER_ID)
      marker.setAttribute(IMarker.CHAR_START, range.start)
      marker.setAttribute(IMarker.CHAR_END, range.end)

    } catch {
      case e: CoreException =>
    }

  }

  def deleteMarkers(resource: IResource, marker_id: String) {
    try {
      resource.deleteMarkers(marker_id, true, IResource.DEPTH_INFINITE)
    } catch {
      case e: CoreException =>
    }
  }
}
