package de.isse.kiv.source

import org.eclipse.jface.text.ITextHover
import org.eclipse.jface.text.IRegion
import org.eclipse.jface.text.ITextViewer
import org.eclipse.jface.text.Region
import de.isse.kiv.resources.ResourceLookup
import de.isse.kiv.resources.ResourceProperties._
import de.isse.kiv.ui.Console
import kiv.expr.Xov
import kiv.expr.Op
import org.eclipse.ui.texteditor.ITextEditor
import kiv.parser.Terminal
import kiv.parser.Terminals
import scala.beans.BeanProperty
import kiv.parser._
import kiv.prog._
import de.isse.kiv.source.parser.VariableToken
import de.isse.kiv.source.parser.SortDefinitionToken
import de.isse.kiv.source.parser.SpecificationNameToken
import de.isse.kiv.source.parser.ProcedureToken
import de.isse.kiv.source.parser.OperationToken
import de.isse.kiv.source.parser.OperationDefinitionToken
import de.isse.kiv.source.parser.KIVToken

class TypeHover(val editor: ITextEditor) extends ITextHover with WithSpec {
  var hoverInfo: String = _

  def getHoverInfo(textViewer: ITextViewer, hoverRegion: IRegion): String = {
    hoverInfo
  }

  import kiv.printer.prettyprint.xpp
  import TokenMap.{ SORT, OP, VARIABLE, PROC }

  def buildHover(kivToken: KIVToken) = {
    val projectModel = file.getProject.projectModel
    val sig = projectModel.signatureAfter(file)
    val tok = kivToken.eclipseToken
    val text = kivToken.stringPresentation

    val info = kivToken match {
      case SpecificationNameToken(strloc) =>
        Some(s"Specification\n\t${strloc.str}")
      case SortDefinitionToken(tycodef) =>
        Some(s"Sort\n\t${xpp(tycodef.symloc.sym)}")
      case VariableToken(prexov: PreXov) =>
        Some(s"Variable\n\t${text}: ${xpp(prexov.xov.typ)}")
      case ProcedureToken(preproc) =>
        Some(s"Procedure\n\t${text}: ${xpp(preproc.proc.mode)}")
      case OperationToken(preop) =>
        Some(s"Operation\n\t${text}: ${xpp(preop.op.typ)}")
      case OperationDefinitionToken(opdef) =>
        Some(s"Operation\n\t${text}")
      case _ =>
        None
    }

    // TODO: remove at some point
    val oldInfo = tok match {
      case SORT =>
        Some("sorts", List(text))
      case OP =>
        Some("operations", sig.oplist.collect {
          case Op(Symbol(`text`), typ, _, _) => text + ": " + xpp(typ)
        })
      case VARIABLE =>
        Some("variables", sig.xovlist.collect {
          case Xov(Symbol(`text`), typ, _) => text + ": " + xpp(typ)
        })
      case PROC =>
        val proc = projectModel.findProc(file, text)
        val procdecl = proc.map { projectModel.getReducedDeclaration(file, _) }.flatten

        val list = procdecl match {
          case Some((procdecl, reducedfpl)) =>
            (procdecl.declprocdecl.proc.procsym.name + " " + xpp(reducedfpl)) :: Nil
          case None =>
            proc match {
              case Some(proc) =>
                (proc.procsym.name + " " + xpp(proc.mode)) :: Nil
              case None =>
                Nil
            }
        }
        Some("procedures", list)
      case _ =>
        None
    }
    info match {
      case Some(string) =>
        hoverInfo = string
      case None => {
        oldInfo match {
          case None                       => hoverInfo = null
          case Some((name, descriptions)) => hoverInfo = name + descriptions.mkString("\n\t", "\n\t", "")
        }
      }
    }
  }

  def getHoverRegion(textViewer: ITextViewer, offset: Int): IRegion = {
    fileModel.tokenAt(offset) match {
      case Some(kivToken) =>
        buildHover(kivToken)
        new Region(kivToken.location.position, kivToken.location.length)
      case None =>
        null
    }
  }
}
