package de.isse.kiv.source.parser

import kiv.spec._
import kiv.prog._
import kiv.parser._
import kiv.util.ScalaExtensions._

class SequentsAnalyzer(val theoremlist: List[Theorem], override val procnameMap: Map[String, AnyProc]) extends SpecMapsAnalyzer {

  private val listExpr = getAllPreExprFromTheoremList(theoremlist)

  def typesMap = Map[Range,TyCodef]()

  def pretycoMap = Map[Range, PreTyCo]()

  def opdefMap = Map[Range, Opdef]()

  override lazy val theoremMap: Map[Range, StringAndLocation] = {
    theoremlist.map { theorem =>
      (theorem.location.toRange, StringAndLocation(theorem.theoremname, theorem.location))
    }.toMap[Range, StringAndLocation]
  }

  override lazy val prexovMap: Map[Range, PreAnyXov] = {
    getAllPreXov(listExpr).map {
      prexov =>
        (prexov.location.get.toRange -> prexov)
    }.toMap
  }

  override lazy val procMap: Map[Range, PreProc] = {
    getAllPreProc(listExpr).flatMap { preproc =>
      preproc.location match {
        case Some(location) =>
          Some(location.toRange -> preproc)
        case None =>
          de.isse.kiv.ui.Console.error.println("Location from anyproc: " + preproc + " is missing")
          None
      }
    }.toMap
  }

  override lazy val preopMap: Map[Range, PreOp] = {
    getAllPreOp(listExpr).flatMap {
      preop =>
        preop.location match {
          case Some(location) =>
            Some(location.toRange -> preop)
          case None =>
            // de.isse.kiv.ui.Console.error.println("Location from preop: "+preop+" is missing!")
            None
        }
    }.toMap
  }

  private def getAllPreExprFromTheoremList(theoremlist: List[Theorem]) = {
    theoremlist.flatMap { theorem => getAllPreExprFromTheorem(theorem) }
  }

  private def getAllPreExprFromTheorem(theorem: Theorem): List[PreExpr] = {
    if (theorem.preseq == null)
      Nil
    else theorem.preseq.patsuc match {
      case prefl1: PreFl1 => prefl1.patfmalist1 ++ theorem.preseq.patant.asInstanceOf[PreFl1].patfmalist1;
      case prefl3: PreFl3 => prefl3.patfmalist1 ++ prefl3.patfmalist2 ++ (theorem.preseq.patant match {
        case prefl1: PreFl1 => prefl1.patfmalist1
        case prefl3: PreFl3 => prefl3.patfmalist1 ++ prefl3.patfmalist2
        case _              => Nil
      })
      case _ => Nil
    }
  }
}
