package de.isse.kiv.ui.navigator

import org.eclipse.ui.navigator.CommonNavigator
import org.eclipse.core.resources.ResourcesPlugin
import org.eclipse.jface.viewers.ITreeContentProvider
import org.eclipse.core.resources.IResourceChangeEvent
import org.eclipse.core.resources.IResourceChangeListener
import org.eclipse.core.resources.IFile
import org.eclipse.ui.PlatformUI
import org.eclipse.core.resources.IProject
import org.eclipse.jface.viewers.Viewer
import org.eclipse.core.resources.IWorkspaceRoot
import de.isse.kiv.project.Nature
import org.eclipse.core.resources.IFolder
import org.eclipse.ui.IWorkbenchPartReference
import org.eclipse.ui.IPartListener2
import org.eclipse.swt.widgets.Composite
import org.eclipse.jface.action.IAction
import org.eclipse.jface.action.Action
import org.eclipse.ui.IEditorPart
import de.isse.kiv.ui.Console
import org.eclipse.jface.resource.ImageDescriptor
import org.eclipse.ui.IEditorInput
import org.eclipse.ui.ide.ResourceUtil
import org.eclipse.jface.action.IContributionItem
import org.eclipse.jface.action.ToolBarContributionItem
import org.eclipse.jface.action.ActionContributionItem
import org.eclipse.swt.SWT
import org.eclipse.jface.viewers.ISelectionChangedListener
import org.eclipse.swt.widgets.Listener
import org.eclipse.swt.widgets.Event
import org.eclipse.swt.widgets.Button
import org.eclipse.swt.widgets.ToolBar
import org.eclipse.ui.internal.navigator.actions.LinkEditorAction
import com.sun.org.apache.xml.internal.serializer.ToStream
import org.eclipse.jface.util.IPropertyChangeListener
import org.eclipse.jface.util.PropertyChangeEvent
import org.eclipse.jface.viewers.TreeSelection
import org.eclipse.jface.viewers.StructuredSelection
import org.eclipse.jface.viewers.ISelection

object Navigator {
  val EMPTY_ARRAY = Array.empty[AnyRef]
  val NAVIGATOR_VIEW_ID = "de.isse.kiv.ui.navigator.view"
  val fileNames = Set("specification.utf8", "sequents.utf8")
}

class Navigator extends CommonNavigator {

  override def getInitialInput = {
    ResourcesPlugin.getWorkspace.getRoot
  }

  var linking: Boolean = false

  object pageListener extends IPartListener2 {
    def activate(ref: IWorkbenchPartReference) {
      if (ref.getPart(true) == Navigator.this)
        editorActivated()
    }

    def partActivated(ref: IWorkbenchPartReference) {
      if (ref.getPart(true).isInstanceOf[IEditorPart])
        editorActivated()
    }

    def partBroughtToTop(ref: IWorkbenchPartReference) { activate(ref) }
    def partClosed(ref: IWorkbenchPartReference) {}
    def partDeactivated(ref: IWorkbenchPartReference) {}
    def partOpened(ref: IWorkbenchPartReference) { activate(ref) }
    def partVisible(ref: IWorkbenchPartReference) { activate(ref) }
    def partHidden(ref: IWorkbenchPartReference) {}
    def partInputChanged(ref: IWorkbenchPartReference) {}
  }
  
  //getAction().setChecked(true)
  override def createPartControl(parent: Composite) {
    //Navigator erzeugen  
    super.createPartControl(parent)

    //Listener in Rückrichtung
    getSite.getPage.addPartListener(pageListener)

    val items = getViewSite.getActionBars.getToolBarManager.getItems

    items.map {
      case toggleButton: ActionContributionItem =>
        val action = toggleButton.getAction
        if (action.isInstanceOf[LinkEditorAction]) {
          linking = action.isChecked

          action.addPropertyChangeListener(new IPropertyChangeListener {
            def propertyChange(event: PropertyChangeEvent) {
              linking = action.isChecked()
              editorActivated()
            }
          })
        }
      case _ =>
    }
  }

  override def dispose() {
    getSite.getPage.removePartListener(pageListener);
  }

  def editorActivated() {
    val editor = getSite.getPage.getActiveEditor
    if(editor == null)
      return

    if (!linking || !getViewSite.getPage.isPartVisible(this))
      return

    val input = editor.getEditorInput;
    val file = ResourceUtil.getFile(input);
    if (file == null)
      return
      
    val selection = new StructuredSelection(file)
    this.selectReveal(selection)
  }

  override def setFocus() {
  }
}

