package de.isse.kiv.ui.navigator

import org.eclipse.jface.viewers.ITreeContentProvider
import org.eclipse.core.resources.IResourceChangeEvent
import org.eclipse.core.resources.IResourceChangeListener
import org.eclipse.core.resources.ResourcesPlugin
import org.eclipse.core.resources.IWorkspaceRoot
import org.eclipse.core.resources.IFile
import org.eclipse.ui.PlatformUI
import org.eclipse.core.resources.IProject
import org.eclipse.jface.viewers.Viewer
import org.eclipse.core.resources.IFolder
import de.isse.kiv.project.Nature
import org.eclipse.core.resources.IResource
import org.eclipse.core.resources.IContainer
import org.eclipse.ltk.internal.core.refactoring.resource.ResourceProcessors
import de.isse.kiv.resources.ResourceProperties
import de.isse.kiv.resources.FileList

class NavigatorContentProvider extends ITreeContentProvider with IResourceChangeListener {
  private var viewer: Viewer = _

  override def inputChanged(viewer: Viewer, oldInput: AnyRef, newInput: AnyRef) {
    this.viewer = viewer
  }

  object refreshViewer extends Runnable {
    override def run { viewer.refresh }
  }

  override def resourceChanged(event: IResourceChangeEvent) {
    PlatformUI.getWorkbench.getDisplay.asyncExec(refreshViewer)
  }

  ResourcesPlugin.getWorkspace.addResourceChangeListener(this, IResourceChangeEvent.POST_CHANGE)

  override def dispose() {
    ResourcesPlugin.getWorkspace.removeResourceChangeListener(this)
  }

  def getParent(element: AnyRef) = {
    element match {
      case file: IFile =>
        file.getProject
      case _ =>
        null
    }
  }

  def hasChildren(element: AnyRef) = {
    element match {
      case root: IWorkspaceRoot => true
      case project: IProject => project.isOpen()
      case _ => false
    }
  }

  def getElements(inputElement: AnyRef) = getChildren(inputElement)

  private def isKIVProject(project: IProject) = {
    if (project.isOpen()) {
      project.hasNature(Nature.NATURE_ID)
    } else {
      /*
      val file = project.getFile(ResourceProperties.DEVGRAPH)
      // file.isLinked returns false
      val path = file.getLocation
      path.toFile.exists
      */
      true
    }
  }

  def getChildren(parent: AnyRef): Array[AnyRef] = {
    parent match {
      case root: IWorkspaceRoot =>
        root.getProjects.collect {
          case project if isKIVProject(project) =>
            project: AnyRef
        }
      case project: IProject if project.isOpen =>
        val files = FileList.collect(project).filter(Navigator.fileNames contains _.getName)
        files.toArray
      case _ =>
        Navigator.EMPTY_ARRAY
    }
  }//geto projectmodel resourceprop -> devgraph -> getspecnames
}