package de.isse.kiv.ui.outline

import org.eclipse.jface.viewers.ITreeContentProvider
import org.eclipse.jface.viewers.TreeViewer
import org.eclipse.jface.viewers.Viewer
import de.isse.kiv.resources.FileModel
import de.isse.kiv.resources.FileModel.RegionString
import kiv.spec.Theorem
import kiv.spec.Spec
import kiv.basic.Typeerror

object SpecOutlineContentProvider {
  val regions: Array[AnyRef] = Array("axioms", "declarations")
}

class SpecOutlineContentProvider extends FileOutlineContentProvider {
  import NullTreeContentProvider._
  import SpecOutlineContentProvider._

  def getElements(element: AnyRef): Array[AnyRef] = {
    getChildren(element)
  }

  def getChildren(element: AnyRef): Array[AnyRef] = {
    val spec = if(model != null) model.getPreSpecificationOrSequents else None
    if (model == null)
      EMPTY
    else if (element eq model)
      regions
    else if (spec == None || spec.get.isRight)
      EMPTY
    else if (element == "axioms")
      try { (spec: @unchecked) match {
        case Some(Left(prespec)) => prespec.theorems.toArray 
        case _ => model.getSpec match {
          case Some(spec) => spec.theoremlist.toArray
          case None => null
        }
        } 
      }  catch { case _: Typeerror => Array() }
    else if (element == "declarations")
      try { (spec: @unchecked) match {
        case Some(Left(prespec)) => prespec.declarations.toArray
        case _ => model.getSpec match {
          case Some(spec) => spec.decllist.toArray
          case None => null
        }
          }
       } catch { case _: Typeerror => Array() }
    else
      EMPTY
  }

  def hasChildren(element: AnyRef): Boolean = {
    (element eq model) || (regions contains element)
  }
}

object TheoremsOutlineContentProvider {
  val regions: Array[AnyRef] = Array("theorems")
}
class TheoremsOutlineContentProvider extends FileOutlineContentProvider {
  import NullTreeContentProvider._
  import TheoremsOutlineContentProvider._

  def getElements(element: AnyRef): Array[AnyRef] = {
    getChildren(element)
  }

  def getChildren(element: AnyRef): Array[AnyRef] = {
    if (model == null)
      EMPTY
    else if (element eq model)
      regions
    else if (element == "theorems")
      model.theorems.toArray
    else
      EMPTY
  }

  def hasChildren(element: AnyRef): Boolean = {
    (element eq model) || (regions contains element)
  }
}

abstract class FileOutlineContentProvider extends ITreeContentProvider with (FileModel => Unit) {

  protected var viewer: Viewer = _
  protected var model: FileModel = _

  override def inputChanged(viewer: Viewer, oldInput: AnyRef, newInput: AnyRef) {
    if (model != null) model.observers -= this
    this.viewer = viewer
    model = newInput.asInstanceOf[FileModel]
    if (model != null) model.observers += this
    this.viewer.getControl.getDisplay.asyncExec(refreshViewer)
  }

  override def dispose() {
    if (model != null) model.observers -= this
  }

  object refreshViewer extends Runnable {
    override def run {
      if (viewer != null && ! viewer.getControl().isDisposed()) {
        viewer.refresh
        viewer.asInstanceOf[TreeViewer].expandAll
      }
    }
  }

  // pfft
  def getParent(element: AnyRef): AnyRef = null

  def apply(model: FileModel) {
    this.model = model
    this.viewer.getControl.getDisplay.asyncExec(refreshViewer)
  }
}

object NullTreeContentProvider extends ITreeContentProvider {
  val EMPTY: Array[AnyRef] = Array()
  def getChildren(parent: AnyRef) = EMPTY
  def getElements(element: AnyRef) = EMPTY
  def getParent(element: AnyRef) = null
  def hasChildren(element: AnyRef) = false

  override def inputChanged(viewer: Viewer, oldInput: AnyRef, newInput: AnyRef) {}
  override def dispose {}
}
