package de.isse.kiv.ui.viewers

import org.eclipse.jface.text.IDocument
import org.eclipse.jface.text.ITextSelection
import org.eclipse.jface.viewers.ISelection
import org.eclipse.jface.viewers.ISelectionProvider
import org.eclipse.swt.SWT
import org.eclipse.swt.custom.ScrolledComposite
import org.eclipse.swt.layout.RowLayout
import org.eclipse.swt.widgets.Button
import org.eclipse.swt.widgets.Composite
import org.eclipse.swt.widgets.Event
import org.eclipse.swt.widgets.Label
import org.eclipse.swt.widgets.Listener
import org.eclipse.ui.IEditorPart
import org.eclipse.ui.PlatformUI
import org.eclipse.ui.part.ViewPart
import org.eclipse.ui.texteditor.ITextEditor
import de.isse.kiv.resources.ResourceLookup
import SymbolView._
import kiv.converter.KivFont
//remove if not needed
import scala.collection.JavaConversions._

object SymbolView {
  val SYMBOL_VIEW_ID = "de.isse.kiv.ui.views.symbols"
}

class SymbolView extends ViewPart with Listener {
  
  private val symbolNames = {
    for (kc <- KivFont.getKivChars if kc != null && kc.names != null) 
      yield (kc.unicode.toString, kc.names)
  }
  
  private def getSymbolTooltip(sym: String): Option[String] = {
    val namesOpt = symbolNames.find(_._1 == sym)
    if (namesOpt.isDefined) {
      val names = (namesOpt.get._2) map ("\\" + _)
      val toolTip = names.mkString("\n")
      Some(toolTip)
    } else Option.empty
  }
  
  val lines = Array(
    ("logic", "→←↑↓↔¬∧∨⊦"),
    ("num", "≠≤≥≡∞∫"),
    ("greek", "αβγδεζηθικλμνξπρστφχψω"),
    ("Greek", "ΓΔΛΞΠΣΦΨΩ"),
    ("set", "∀∃×∅∈∋⊆⊇⊂⊃∪∩"),
    ("sem", "⇔⇒⊧⇐⟦⟧⊤⊥"),
    ("dom", "⊑⊒⊏⊐⊔⊓"),
    ("dl", "⟨⟩⦉⦊"),
    ("tl", "∥○□◇●"),
    ("misc", "⌈⌉⌊⌋≪≫⌜⌝⊕⊖⊗⊘⊙△"))

  override def createPartControl(parent: Composite) {
    val sc = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL | SWT.BORDER)
    val c = new Composite(sc, SWT.NONE)
    sc.setContent(c)
    val vl = new RowLayout()
    vl.`type` = SWT.VERTICAL
    c.setLayout(vl)
    for ((name, symbols) <- lines) {
      val d = new Composite(c, SWT.NONE)
      val hl = new RowLayout()
      hl.`type` = SWT.HORIZONTAL
      hl.wrap = true
      d.setLayout(hl)
      val l = new Label(d, SWT.LEFT)
      l.setText(name)
      for (sym <- symbols) {
        val b = new Button(d, SWT.PUSH)
        b.setText(sym.toString)
        b.addListener(SWT.MouseUp, this)
        val toolTipOpt = getSymbolTooltip(sym.toString)
        if (toolTipOpt.isDefined)
          b.setToolTipText(toolTipOpt.get)
      }
      d.pack()
    }
    c.pack()
    c.setSize(c.computeSize(SWT.DEFAULT, SWT.DEFAULT))
  }

  override def setFocus() {
  }

  override def handleEvent(event: Event) {
    try {
      PlatformUI.getWorkbench.getActiveWorkbenchWindow.getActivePage.getActiveEditor match {
        case editor: ITextEditor =>
          editor.getSelectionProvider.getSelection match {
            case selection: ITextSelection =>
              val offset = selection.getOffset
              val length = selection.getLength
              val doc = ResourceLookup.document
              val text = event.widget.asInstanceOf[Button].getText
              doc.replace(offset, length, text)
              // editor.selectAndReveal(offset + 1, 0)
              // editor.setFocus()
          }
      }
    } catch {
      case e: Exception =>
    }
  }
}
