package de.isse.kiv.ui.wizards

import org.eclipse.jface.wizard.Wizard
import org.eclipse.ui._
import org.eclipse.core.runtime._
import org.eclipse.jface.viewers.IStructuredSelection
import org.eclipse.core.resources.IProject
import java.util.zip._
import java.io.FileOutputStream
import org.eclipse.core.resources._
import java.nio.file.Files
import java.nio.charset.StandardCharsets
import org.eclipse.jface.dialogs.MessageDialog

class ExportProjectWizard extends Wizard with INewWizard with IExecutableExtension {

  private var page: ExportProjectWizardPage = _
  private var project: IProject = _
  private var configElement: IConfigurationElement = _

  override def init(workbench: IWorkbench, selection: IStructuredSelection) {
    project = selection.getFirstElement.asInstanceOf[IProject]
    project.refreshLocal(IResource.DEPTH_INFINITE, null)
  }

  override def addPages() {
    page = new ExportProjectWizardPage(project)
    addPage(page)
  }

  override def setInitializationData(config: IConfigurationElement, propertyName: String, data: AnyRef) {
    this.configElement = config
  }

  override def performFinish(): Boolean = {
    val path = page.getPath
    
    if (path.toFile.exists()) {
      val dialog = new MessageDialog(null, "Overwrite Archive?", null, "The archive \"" + path + "\" already exists. Should it be overrwritten?",
                                     MessageDialog.QUESTION, Array[String]("Yes", "No"), 0)
      if (dialog.open() != 0)
        return false
    }

    val file = new FileOutputStream(path.toFile())
    val zipfile = new ZipOutputStream(file, StandardCharsets.UTF_8)
    addToZipFile(project, Path.EMPTY.append(project.getName))
    def addToZipFile(container: IResource, relative: IPath) {
      container match {
        case file: IFile =>
          if (file.getName != ".project") {
            zipfile.putNextEntry(new ZipEntry(relative.append(file.getName).toString))
            val contents = Files.readAllBytes(file.getLocation.toFile().toPath())
            zipfile.write(contents)
          }
        case folder: IFolder =>
          if (folder.getName != ".settings")
            folder.members.foreach { addToZipFile(_, relative.append(folder.getName)) }
        case project: IProject =>
          project.members.foreach { addToZipFile(_, relative) }
      }
    }
    zipfile.close()
    file.close()
    true
  }
}
