package de.isse.kiv.ui.wizards

import java.io.File
import org.eclipse.core.resources.IWorkspace
import org.eclipse.core.resources.ResourcesPlugin
import org.eclipse.core.runtime.IPath
import org.eclipse.ui.dialogs.WizardNewProjectCreationPage
import de.isse.kiv.resources.ResourceProperties

class NewProjectWizardPage extends WizardNewProjectCreationPage("Create new KIV Project") {

  setTitle("Create new KIV Project")

  setDescription("Specify the name of the new project.\n" + "Enter a project location if you want the project files to be located outside the workspace.")

  def getProjectLocationPath(): IPath = {
    val name = getProjectName
    var location = getLocationPath
    val workspace = ResourcesPlugin.getWorkspace
    val rootPath = workspace.getRoot.getLocation
    if (location == rootPath) location = location.append(name)
    location
  }

  override def validatePage(): Boolean = {
    val ok = super.validatePage()
    if (!ok) return false
    val location = getProjectLocationPath
    val devgraph_path = location.append(ResourceProperties.DEVGRAPH)
    val devgraph_file = devgraph_path.toFile()
    if (devgraph_file.exists() && devgraph_file.isFile) {
      setErrorMessage("The specified directory already contains a KIV project.\n" + "Use the KIV import feature instead.")
      return false
    }
    val file = location.toFile()
    if (location.toFile().exists() && file.isDirectory && file.list().length != 0) {
      setErrorMessage("The specified directory is not empty.")
      return false
    }
    true
  }
}
