package de.isse.kiv.ui.wizards

import org.eclipse.jface.dialogs.Dialog
import org.eclipse.ltk.ui.refactoring.UserInputWizardPage
import org.eclipse.swt.SWT
import org.eclipse.swt.events.ModifyEvent
import org.eclipse.swt.events.ModifyListener
import org.eclipse.swt.events.SelectionEvent
import org.eclipse.swt.events.SelectionListener
import org.eclipse.swt.layout.GridData
import org.eclipse.swt.layout.GridLayout
import org.eclipse.swt.widgets.Button
import org.eclipse.swt.widgets.Composite
import org.eclipse.swt.widgets.Label
import org.eclipse.swt.widgets.Text

import de.isse.kiv.refactoring.CodeRenameRefactoring

class RenameRefactoringWizardPage(name: String, renameWizard: RenameRefactoringWizard) extends UserInputWizardPage(name: String) {

  var initialized = false
  var renameField: Text = _  

  def createControl(parent: Composite): Unit = {

    initializeDialogUnits(parent)
    val composite = new Composite(parent, SWT.NONE)
    setControl(composite)
    val gridLayout = new GridLayout(1, false)
    gridLayout.verticalSpacing = 8
    composite.setLayout(gridLayout)

  }

  override def setVisible(visible: Boolean) {
    if (visible && !initialized) {
      initialized = true
      val composite = getControl.asInstanceOf[Composite]
      createUserInterface(composite)
      Dialog.applyDialogFont(composite)
    }
    super.setVisible(visible)
  }

  def createUserInterface(composite: Composite) {

    val gridData = new GridData(SWT.FILL, SWT.FILL, true, false)

    gridData.verticalIndent = 8
    composite.setLayoutData(gridData)
    val gridLayout2 = new GridLayout(1, false)
    gridLayout2.verticalSpacing = 8
    gridLayout2.marginTop = 3
    composite.setLayout(gridLayout2)

    val label = new Label(composite, SWT.NONE)
    label.setText("New Name:")
    renameField = new Text(composite, SWT.SINGLE | SWT.LEFT | SWT.BORDER)
    renameField.setLayoutData(new GridData(GridData.FILL_HORIZONTAL))

    val updateReferenceCheckBox = new Button(composite, SWT.CHECK)
    updateReferenceCheckBox.setLayoutData(new GridData(SWT.LEFT, SWT.TOP, true, false))
    updateReferenceCheckBox.setText("Update reference?")
    updateReferenceCheckBox.setSelection(true)
    updateReferenceCheckBox.setEnabled(true)
    
    val refactorer = renameWizard.getRefactoring.asInstanceOf[CodeRenameRefactoring]
    
    renameField.addModifyListener(new ModifyListener() {

      override def modifyText(event: ModifyEvent) {
        refactorer.editedString = renameField.getText
      }
    })
    
    refactorer.updateReferences = updateReferenceCheckBox.getSelection
    
    updateReferenceCheckBox.addSelectionListener(new SelectionListener() {
      
      override def widgetSelected(event: SelectionEvent) {
        refactorer.updateReferences = updateReferenceCheckBox.getSelection
      }
      override def widgetDefaultSelected(event: SelectionEvent) {
        refactorer.updateReferences = updateReferenceCheckBox.getSelection
      }
    })

    
    renameField.setText(refactorer.selectedTokenString)
    renameField.setFocus()
    renameField.selectAll()

  }

}